@@ -35,8 +35,9 @@ from .commands.cmake import get_installed_files
 from .commands.make import MAKE_EXEC
 
 from .utils import copyfiles
-from .utils import get_env_loader
+from .utils import loadenv
 from .utils import makedirs
+from .utils import require_command
 from .utils import rmfiles
 
 
@@ -243,14 +244,33 @@ def link_devel_products(
 
         # create directories in the destination develspace
         for dirname in dirs:
+            source_dir = os.path.join(source_path, dirname)
             dest_dir = os.path.join(dest_path, dirname)
 
-            if not os.path.exists(dest_dir):
-                # Create the dest directory if it doesn't exist
-                os.mkdir(dest_dir)
-            elif not os.path.isdir(dest_dir):
-                logger.err('Error: Cannot create directory: ' + dest_dir)
-                return -1
+            if os.path.islink(source_dir):
+                # Store the source/dest pair
+                products.append((source_dir, dest_dir))
+
+                if os.path.exists(dest_dir):
+                    if os.path.realpath(dest_dir) != os.path.realpath(source_dir):
+                        files_that_collide.append(dest_dir)
+                    else:
+                        logger.out('Linked: ({}, {})'.format(source_dir, dest_dir))
+                else:
+                    # Create a symlink
+                    logger.out('Symlinking %s' % (dest_dir))
+                    try:
+                        os.symlink(source_dir, dest_dir)
+                    except OSError:
+                        logger.err('Could not create symlink `{}` referencing `{}`'.format(dest_dir, source_dir))
+                        raise
+            else:
+                if not os.path.exists(dest_dir):
+                    # Create the dest directory if it doesn't exist
+                    os.mkdir(dest_dir)
+                elif not os.path.isdir(dest_dir):
+                    logger.err('Error: Cannot create directory: {}'.format(dest_dir))
+                    return -1
 
         # create symbolic links from the source to the dest
         for filename in files:
@@ -269,8 +289,8 @@ def link_devel_products(
             if os.path.exists(dest_file):
                 if os.path.realpath(dest_file) != os.path.realpath(source_file):
                     # Compute hashes for colliding files
-                    source_hash = md5(open(os.path.realpath(source_file)).read().encode('utf-8')).hexdigest()
-                    dest_hash = md5(open(os.path.realpath(dest_file)).read().encode('utf-8')).hexdigest()
+                    source_hash = md5(open(os.path.realpath(source_file), "rb").read()).hexdigest()
+                    dest_hash = md5(open(os.path.realpath(dest_file), "rb").read()).hexdigest()
                     # If the link links to a different file, report a warning and increment
                     # the collision counter for this path
                     if dest_hash != source_hash:
@@ -308,7 +328,8 @@ def link_devel_products(
     # after this build, and update the collision file
     try:
         clean_linked_files(logger, event_queue, metadata_path, files_that_collide, files_to_clean, dry_run=False)
-    except:
+    except:  # noqa: E722
+        # Silencing E722 here since we immediately re-raise the exception.
         logger.err('Could not clean linked files.')
         raise
 
@@ -350,10 +371,23 @@ def create_catkin_build_job(context, package, package_path, dependencies, force_
     install_space = context.package_install_space(package)
     # Package metadata path
     metadata_path = context.package_metadata_path(package)
+    # Environment dictionary for the job, which will be built
+    # up by the executions in the loadenv stage.
+    job_env = dict(os.environ)
 
     # Create job stages
     stages = []
 
+    # Load environment for job.
+    stages.append(FunctionStage(
+        'loadenv',
+        loadenv,
+        locked_resource=None if context.isolate_install else 'installspace',
+        job_env=job_env,
+        package=package,
+        context=context
+    ))
+
     # Create package build space
     stages.append(FunctionStage(
         'mkdir',
@@ -400,6 +434,8 @@ def create_catkin_build_job(context, package, package_path, dependencies, force_
             prefix=context.package_dest_path(package)
         ))
 
+        require_command('cmake', CMAKE_EXEC)
+
         # CMake command
         stages.append(CommandStage(
             'cmake',
@@ -441,6 +477,8 @@ def create_catkin_build_job(context, package, package_path, dependencies, force_
             cwd=build_space,
         ))
 
+    require_command('make', MAKE_EXEC)
+
     # Make command
     stages.append(CommandStage(
         'make',
@@ -472,13 +510,13 @@ def create_catkin_build_job(context, package, package_path, dependencies, force_
             [MAKE_EXEC, 'install'],
             cwd=build_space,
             logger_factory=CMakeMakeIOBufferProtocol.factory,
-            locked_resource='installspace'
+            locked_resource=None if context.isolate_install else 'installspace'
         ))
 
     return Job(
         jid=package.name,
         deps=dependencies,
-        env_loader=get_env_loader(package, context),
+        env=job_env,
         stages=stages)
 
 
@@ -499,6 +537,8 @@ def create_catkin_clean_job(
     build_space = context.package_build_space(package)
     # Package metadata path
     metadata_path = context.package_metadata_path(package)
+    # Environment dictionary for the job, empty for a clean job
+    job_env = {}
 
     # Remove installed files
     if clean_install:
@@ -566,7 +606,7 @@ def create_catkin_clean_job(
     return Job(
         jid=package.name,
         deps=dependencies,
-        env_loader=get_env_loader(package, context),
+        env=job_env,
         stages=stages)
 
 
@@ -599,6 +639,9 @@ DEVEL_LINK_BLACKLIST = DEVEL_LINK_PREBUILD_BLACKLIST + [
     'setup.bash',
     'setup.zsh',
     'setup.sh',
+    'local_setup.bash',
+    'local_setup.zsh',
+    'local_setup.sh',
     '_setup_util.py',
 ]
 
